@@ -1,4 +1,4 @@
-/*  Copyright (C) 2005-2016, Axis Communications AB, LUND, SWEDEN
+/*  Copyright (C) 2005-2018, Axis Communications AB, LUND, SWEDEN
  *
  *  This file is part of RAPP.
  *
@@ -178,6 +178,12 @@ rapp_bmark_exec_bin_bin_iip(int (*func)(), const int *args);
 static void
 rapp_bmark_exec_u8_bin(int (*func)(), const int *args);
 
+static void
+rapp_bmark_exec_u8_bin_i(int (*func)(), const int *args);
+
+static void
+rapp_bmark_exec_u8_bin_ii(int (*func)(), const int *args);
+
 static void
 rapp_bmark_exec_bin_u8(int (*func)(), const int *args);
 
@@ -275,10 +281,10 @@ static const rapp_bmark_table_t rapp_bmark_suite[] = {
     RAPP_BMARK_ENTRY(type_u8_to_bin,  NULL,  u8_bin, 0, 0),
     RAPP_BMARK_ENTRY(type_bin_to_u8,  NULL,  bin_u8, 0, 0),
     /* rapp_thresh functions */
-    RAPP_BMARK_ENTRY(thresh_gt_u8,   NULL, u8_bin, 7, 0),
-    RAPP_BMARK_ENTRY(thresh_lt_u8,   NULL, u8_bin, 7, 0),
-    RAPP_BMARK_ENTRY(thresh_gtlt_u8, NULL, u8_bin, 7, 9),
-    RAPP_BMARK_ENTRY(thresh_ltgt_u8, NULL, u8_bin, 7, 9),
+    RAPP_BMARK_ENTRY(thresh_gt_u8,   NULL, u8_bin_i, 7, 0),
+    RAPP_BMARK_ENTRY(thresh_lt_u8,   NULL, u8_bin_i, 7, 0),
+    RAPP_BMARK_ENTRY(thresh_gtlt_u8, NULL, u8_bin_ii, 7, 9),
+    RAPP_BMARK_ENTRY(thresh_ltgt_u8, NULL, u8_bin_ii, 7, 9),
     RAPP_BMARK_ENTRY(thresh_gt_pixel_u8,   NULL, thresh_pixel, 1, 0),
     RAPP_BMARK_ENTRY(thresh_lt_pixel_u8,   NULL, thresh_pixel, 1, 0),
     RAPP_BMARK_ENTRY(thresh_gtlt_pixel_u8, NULL, thresh_pixel, 2, 0),
@@ -298,7 +304,7 @@ static const rapp_bmark_table_t rapp_bmark_suite[] = {
     RAPP_BMARK_ENTRY(reduce_2x2_rk4_bin, NULL, bin_bin, 0, 0),
     /* rapp_expand_bin functions */
     RAPP_BMARK_ENTRY(expand_1x2_bin, NULL, expand, 0, 0),
-    RAPP_BMARK_ENTRY(expand_2x2_bin, NULL, expand, 0, 0),
+    RAPP_BMARK_ENTRY(expand_2x1_bin, NULL, expand, 0, 0),
     RAPP_BMARK_ENTRY(expand_2x2_bin, NULL, expand, 0, 0),
     /* rapp_rotate_u8 functions */
     RAPP_BMARK_ENTRY(rotate_cw_u8,  NULL, rotate_u8, 0, 0),
@@ -722,12 +728,28 @@ rapp_bmark_exec_bin_u8(int (*func)(), const int *args)
     const rapp_bmark_data_t *data = &rapp_bmark_data;
     (void)args;
     (*func)(data->dst,   data->dim_u8,
-            data->set,   data->dim_bin,
+            /**
+             * We run zeros as source only because that's what tuning
+             * does for rc_type_bin_to_u8, to avoid cache-effects for
+             * the generic implementation. It may matter for other
+             * functions too.
+             */
+            data->clear, data->dim_bin,
             data->width, data->height);
 }
 
 static void
 rapp_bmark_exec_u8_bin(int (*func)(), const int *args)
+{
+    const rapp_bmark_data_t *data = &rapp_bmark_data;
+    (void)args;
+    (*func)(data->dst,   data->dim_bin,
+            data->set,   data->dim_u8,
+            data->width, data->height);
+}
+
+static void
+rapp_bmark_exec_u8_bin_i(int (*func)(), const int *args)
 {
     const rapp_bmark_data_t *data = &rapp_bmark_data;
     (*func)(data->dst,   data->dim_bin,
@@ -735,6 +757,15 @@ rapp_bmark_exec_u8_bin(int (*func)(), const int *args)
             data->width, data->height, args[0]);
 }
 
+static void
+rapp_bmark_exec_u8_bin_ii(int (*func)(), const int *args)
+{
+    const rapp_bmark_data_t *data = &rapp_bmark_data;
+    (*func)(data->dst,   data->dim_bin,
+            data->set,   data->dim_u8,
+            data->width, data->height, args[0], args[1]);
+}
+
 static void
 rapp_bmark_exec_u8(int (*func)(), const int *args)
 {
