@@ -1,4 +1,4 @@
-/*  Copyright (C) 2011-2016, Axis Communications AB, LUND, SWEDEN
+/*  Copyright (C) 2011-2018, Axis Communications AB, LUND, SWEDEN
  *
  *  This file is part of RAPP.
  *
@@ -28,7 +28,7 @@
 /**
  *  @file   rc_vec_neon.h
  *  @brief  RAPP Compute layer vector operations
- *          using the ARM NEON instruction set.
+ *          using the ARM 64-bit NEON instruction set.
  */
 
 #ifndef RC_VEC_NEON_H
@@ -304,12 +304,12 @@ do {                                                        \
  */
 #define RC_VEC_GETMASKV_(maskv, vec)                    \
 do {                                                    \
-    int8x8_t shv_ = {0, -1, -2, -3, -4, -5, -6, -7};    \
-    rc_vec_t tmp_ = vrev64_u8(vec);                     \
-    tmp_ = vand_u8(tmp_, vdup_n_u8(0x80));              \
-    tmp_ = vshl_u8(tmp_, shv_);                         \
+    int8x8_t rcshv_ = {-7, -6, -5, -4, -3, -2, -1, 0};  \
+    rc_vec_t tmp1_, tmp2_;                              \
+    tmp1_ = vand_u8(vec, vdup_n_u8(0x80));              \
+    tmp2_ = vshl_u8(tmp1_, rcshv_);                     \
     (maskv) = (rc_vec_t)                                \
-        vpaddl_u32(vpaddl_u16(vpaddl_u8(tmp_)));        \
+        vpaddl_u32(vpaddl_u16(vpaddl_u8(tmp2_)));       \
 } while (0)
 
 #define RC_VEC_GETMASKW(maskw, vec)         \
