@@ -1,4 +1,4 @@
-/*  Copyright (C) 2005-2010, Axis Communications AB, LUND, SWEDEN
+/*  Copyright (C) 2005-2018, Axis Communications AB, LUND, SWEDEN
  *
  *  This file is part of RAPP.
  *
@@ -381,7 +381,7 @@ do {                                                                    \
 /**
  *  Permutation table with the bit mapping 0,2,4,6,1,3,5,7.
  */
-static const uint8_t rc_reduce_perm_lut[256] = {
+static const uint8_t rc_reduce_perm_lut[256] __attribute__((__unused__)) = {
     0x00, 0x01, 0x10, 0x11, 0x02, 0x03, 0x12, 0x13,
     0x20, 0x21, 0x30, 0x31, 0x22, 0x23, 0x32, 0x33,
     0x04, 0x05, 0x14, 0x15, 0x06, 0x07, 0x16, 0x17,
