@@ -1,4 +1,4 @@
-/*  Copyright (C) 2005-2016, Axis Communications AB, LUND, SWEDEN
+/*  Copyright (C) 2005-2018, Axis Communications AB, LUND, SWEDEN
  *
  *  This file is part of RAPP.
  *
@@ -176,6 +176,12 @@ rc_bmark_exec_u8_bin(int (*func)(), const int *args);
 static void
 rc_bmark_exec_u8_bin_c(int (*func)(), const int *args);
 
+static void
+rc_bmark_exec_u8_bin_i(int (*func)(), const int *args);
+
+static void
+rc_bmark_exec_u8_bin_ii(int (*func)(), const int *args);
+
 static void
 rc_bmark_exec_u8_bin_u8_c(int (*func)(), const int *args);
 
@@ -288,10 +294,10 @@ static const rc_bmark_table_t rc_bmark_suite[] = {
     RC_BMARK_ENTRY(rc_type_u8_to_bin,                     u8_bin,    0, 0),
     RC_BMARK_ENTRY(rc_type_bin_to_u8,                     bin_u8,    0, 0),
     /* Thresholding */
-    RC_BMARK_ENTRY(rc_thresh_gt_u8,                       u8_bin,    0, 0),
-    RC_BMARK_ENTRY(rc_thresh_lt_u8,                       u8_bin,    0, 0),
-    RC_BMARK_ENTRY(rc_thresh_gtlt_u8,                     u8_bin,    0, 0),
-    RC_BMARK_ENTRY(rc_thresh_ltgt_u8,                     u8_bin,    0, 0),
+    RC_BMARK_ENTRY(rc_thresh_gt_u8,                       u8_bin_i,  7, 0),
+    RC_BMARK_ENTRY(rc_thresh_lt_u8,                       u8_bin_i,  7, 0),
+    RC_BMARK_ENTRY(rc_thresh_gtlt_u8,                     u8_bin_ii, 7, 9),
+    RC_BMARK_ENTRY(rc_thresh_ltgt_u8,                     u8_bin_ii, 7, 9),
     RC_BMARK_ENTRY(rc_thresh_gt_pixel_u8,                 thresh_pixel, 1, 0),
     RC_BMARK_ENTRY(rc_thresh_lt_pixel_u8,                 thresh_pixel, 1, 0),
     RC_BMARK_ENTRY(rc_thresh_gtlt_pixel_u8,               thresh_pixel, 2, 0),
@@ -836,6 +842,24 @@ rc_bmark_exec_u8_bin(int (*func)(), const int *args)
             rc_bmark_data.width, rc_bmark_data.height);
 }
 
+static void
+rc_bmark_exec_u8_bin_i(int (*func)(), const int *args)
+{
+    (*func)(rc_bmark_data.dst,   rc_bmark_data.dim_bin,
+            rc_bmark_data.src,   rc_bmark_data.dim_u8,
+            rc_bmark_data.width, rc_bmark_data.height,
+            args[0]);
+}
+
+static void
+rc_bmark_exec_u8_bin_ii(int (*func)(), const int *args)
+{
+    (*func)(rc_bmark_data.dst,   rc_bmark_data.dim_bin,
+            rc_bmark_data.src,   rc_bmark_data.dim_u8,
+            rc_bmark_data.width, rc_bmark_data.height,
+            args[0], args[1]);
+}
+
 static void
 rc_bmark_exec_u8(int (*func)(), const int *args)
 {
