@@ -1,4 +1,4 @@
-/*  Copyright (C) 2005-2011, Axis Communications AB, LUND, SWEDEN
+/*  Copyright (C) 2005-2012, Axis Communications AB, LUND, SWEDEN
  *
  *  This file is part of RAPP.
  *
@@ -320,7 +320,7 @@ do {                                              \
  *  Horizontal Sobel operator, vertical component.
  *  Partly separable. We store the result of column-wise
  *  gaussian filter, computed using AVGT + AVGT and AVGR + AVGT
- *  in the first and second output values, respectivly.
+ *  in the first and second output values, respectively.
  */
 #if defined RC_VEC_AVGT && defined RC_VEC_AVGR
 #define RC_FILTER_SOBEL_HORZ_V(v1x, v2x, v3x, p1, p2, p3) \
@@ -351,7 +351,7 @@ do {                                                    \
  *  Vertical Sobel operator, vertical component.
  *  Partly separable. We store the result of column-wise
  *  difference filter, computed using SUBHT and SUBHR in
- *  the first and second output values, respectivly.
+ *  the first and second output values, respectively.
  */
 #if defined RC_VEC_SUBHT && defined RC_VEC_SUBHR
 #define RC_FILTER_SOBEL_VERT_V(v1x, v2x, v3x, p1, p2, p3) \
@@ -416,7 +416,7 @@ do {                                                        \
 /**
  *  Vertical absolute-value Sobel operator, vertical component.
  *  Partly separable. We store the vertical difference result from
- *  SUBHT and SUBHR in the first and second variables, respectivly.
+ *  SUBHT and SUBHR in the first and second variables, respectively.
  */
 #if defined RC_VEC_SUBHT && defined RC_VEC_SUBHR
 #define RC_FILTER_SOBEL_VERT_ABS_V(v1x, v2x, v3x, p1, p2, p3) \
@@ -758,6 +758,8 @@ do {                                                                     \
         /* Kill compiler warnings */                                     \
         (void)v21, (void)v22;                                            \
         (void)v31, (void)v32;                                            \
+        (void)v23, (void)v33;                                            \
+        (void)s2;                                                        \
                                                                          \
         /* Handle all full periods */                                    \
         for (x = 0; x < len; x++) {                                      \
@@ -813,6 +815,8 @@ do {                                                        \
     RC_VEC_LOAD(p3, &(src)[i3]);                            \
     vert(v13, v23, v33, p1, p2, p3);                        \
     horz(sum, v11, v12, v13, v21, v22, v23, v31, v32, v33); \
+    /* Kill compiler warnings. */                           \
+    (void)p2;                                               \
     RC_VEC_STORE(&(dst)[j], sum);                           \
     (i1) += RC_VEC_SIZE;                                    \
     (i2) += RC_VEC_SIZE;                                    \
