@@ -1,4 +1,4 @@
-/*  Copyright (C) 2005-2011, Axis Communications AB, LUND, SWEDEN
+/*  Copyright (C) 2005-2012, Axis Communications AB, LUND, SWEDEN
  *
  *  This file is part of RAPP.
  *
@@ -83,6 +83,10 @@ do {                                                                          \
     RC_VEC_SPLAT(thr1_, low);                                                 \
     RC_VEC_SPLAT(thr2_, high);                                                \
                                                                               \
+    /* Kill compiler warnings. */                                             \
+    (void)thr1_;                                                              \
+    (void)thr2_;                                                              \
+                                                                              \
     if ((unroll) == 4 &&                       /* Constant */                 \
         (8*RC_WORD_SIZE) / RC_VEC_SIZE >= 4 && /* Constant */                 \
         (blk_ > 0 || end_ >= 4))               /* Variable */                 \
@@ -132,6 +136,10 @@ do {                                                                          \
     RC_VEC_SPLAT(thr1_, low);                                                 \
     RC_VEC_SPLAT(thr2_, high);                                                \
                                                                               \
+    /* Kill compiler warnings. */                                             \
+    (void)thr1_;                                                              \
+    (void)thr2_;                                                              \
+                                                                              \
     if ((unroll) == 4 &&                       /* Constant */                 \
         (blk_ > 0 || end_ >= 4))               /* Variable */                 \
     {                                                                         \
