@@ -1,4 +1,4 @@
-/*  Copyright (C) 2005-2010, Axis Communications AB, LUND, SWEDEN
+/*  Copyright (C) 2005-2018, Axis Communications AB, LUND, SWEDEN
  *
  *  This file is part of RAPP.
  *
@@ -100,8 +100,8 @@ bool
 rapp_test_stat_sum2_u8(void)
 {
     assert(sizeof(int) == sizeof(int32_t));
-    return rapp_test_stat_driver((int32_t (*)())&rapp_stat_sum2_u8,
-                                 (int32_t (*)())&rapp_ref_stat_sum2_u8,
+    return rapp_test_stat_driver(&rapp_stat_sum2_u8,
+                                 &rapp_ref_stat_sum2_u8,
                                  0, false);
 }
 
