@@ -214,6 +214,7 @@ rapp_ref_thresh_pixel_driver(uint8_t *dst, int dst_dim,
 
     for (y = 0; y < height; y++) {
         for (x = 0; x < width; x++) {
+            int bit;
             int val = rapp_pixel_get_u8(src, src_dim, x, y);
 
             int thresh_low = rapp_pixel_get_u8(low, low_dim, x, y);
@@ -223,7 +224,7 @@ rapp_ref_thresh_pixel_driver(uint8_t *dst, int dst_dim,
                 thresh_high = rapp_pixel_get_u8(high, high_dim, x, y);
             }
 
-            int bit = (*cmp)(val, thresh_low, thresh_high);
+            bit = (*cmp)(val, thresh_low, thresh_high);
 
             rapp_pixel_set_bin(dst, dst_dim, 0, x, y, bit);
         }
